import re
from collections import OrderedDict

from inventory_provider.db import ims
from inventory_provider.db.ims import InventoryStatus

INTERNAL_POP_NAMES = {
    'Cambridge OC',
    'DANTE Lab',
    'GÉANT LAB',
    'GEANT LAB',
    'Amsterdam GEANT Office',
    'Amsterdam GÉANT Office'
}


def lookup_lg_routers(ds):
    pattern = re.compile("vpn-proxy|vrr|taas", re.IGNORECASE)

    def _matching_node(node_):
        if InventoryStatus(node_['InventoryStatusId']) not in [
            InventoryStatus.IN_SERVICE,
            InventoryStatus.PLANNED  # remove once data fully migrated
        ]:
            return False

        if pattern.match(node_['Name']):
            return False

        return True

    site_nav_props = [
        ims.SITE_PROPERTIES['SiteAliases'],
        ims.SITE_PROPERTIES['City'],
        ims.SITE_PROPERTIES['Country']
    ]

    eq_definitions = ds.get_filtered_entities(
        'EquipmentDefinition',
        'Name like mx',
        ims.EQUIP_DEF_PROPERTIES['Nodes'])

    for eq_def in eq_definitions:
        nodes = eq_def['Nodes']

        for node in nodes:
            if not _matching_node(node):
                continue

            site = ds.get_entity_by_id('Site', node['SiteId'], site_nav_props,
                                       True)
            city = site['City']

            abbreviation = ''
            try:
                abbreviation = site['SiteAliases'][0]['AliasName']
            except IndexError:
                pass  # no alias - ignore silently

            eq = {
                    'equipment name': node['Name'],
                    'type':
                    'INTERNAL'
                    if site['Name'] in INTERNAL_POP_NAMES
                    else 'CORE',
                    'pop': {
                        'name': site['Name'],
                        'city': city['Name'],
                        'country': city['Country']['Name'],
                        'country code': city['Country']['Abbreviation'],
                        'abbreviation': abbreviation,
                        'longitude': site['Longitude'],
                        'latitude': site['Latitude'],
                    }
                }
            yield(eq)


def lookup_pop_info(ds, hostname):
    site_nav_props = [
        ims.SITE_PROPERTIES['City'],
        ims.SITE_PROPERTIES['SiteAliases'],
        ims.SITE_PROPERTIES['Country']
    ]

    node = ds.get_entity_by_name('Node', hostname)
    if not node:
        return None
    site = ds.get_entity_by_id('Site', node['SiteId'], site_nav_props, True)
    city = site['City']
    abbreviation = ''
    try:
        abbreviation = site['SiteAliases'][0]['AliasName']
    except IndexError:
        pass  # no alias - ignore silently
    eq = {
            'equipment-name': node['Name'],
            'status': InventoryStatus(node['InventoryStatusId']).name,
            'pop': {
                'name': site['Name'],
                'city': city['Name'],
                'country': city['Country']['Name'],
                'abbreviation': abbreviation,
                'longitude': site['Longitude'],
                'latitude': site['Latitude'],
            }
        }
    return eq
